/*
 * Notification support.
 *
 * Copyright (C) 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <set>

#include <systypes/base.h>



/* Notification support, allowing pipe or file users to receive events due to
   activity on those objects. */

class NotificationSupport
{
protected:
    /* Endpoint count. */

    unsigned int _max_endpoints;

    /* Notification endpoints and flags, set up at initialisation for the
       appropriate number of endpoints. */

    std::set<l4_cap_idx_t> *_notifiers;
    notify_flags_t *_flags, *_deferred;

    virtual void release_notifiers();

public:
    explicit NotificationSupport(unsigned int endpoints);

    virtual ~NotificationSupport();

    virtual void notify(unsigned int endpoint, notify_flags_t flags, l4_cap_idx_t source=L4_INVALID_CAP);

    virtual void subscribe(unsigned int endpoint, l4_cap_idx_t notifier, notify_flags_t flags);

    virtual void unsubscribe(unsigned int endpoint, l4_cap_idx_t notifier);
};

// vim: tabstop=4 expandtab shiftwidth=4
